package mcmultipart.block;

import mcmultipart.capabilities.ISlottedCapabilityProvider;
import mcmultipart.capabilities.MultipartCapabilityHelper;
import mcmultipart.microblock.IMicroblock;
import mcmultipart.microblock.IMicroblockContainerTile;
import mcmultipart.microblock.MicroblockContainer;
import mcmultipart.multipart.IMultipart;
import mcmultipart.multipart.PartSlot;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.NetworkManager;
import net.minecraft.network.play.server.SPacketUpdateTileEntity;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;

/**
 * An implementation of {@link TileMultipartContainer} and {@link IMicroblockContainerTile} that acts as a microblock container.<br/>
 * Extend this class if you want a custom TileEntity for your {@link BlockCoverable}.
 */
public class TileCoverable extends TileEntity implements IMicroblockContainerTile, ISlottedCapabilityProvider {

    private MicroblockContainer container;

    public TileCoverable() {

    }

    @Override
    public World getWorldIn() {

        return func_145831_w();
    }

    @Override
    public BlockPos getPosIn() {

        return func_174877_v();
    }

    @Override
    public MicroblockContainer getMicroblockContainer() {

        return container != null ? container : (container = new MicroblockContainer(this));
    }

    @Override
    public boolean canAddMicroblock(IMicroblock microblock) {

        return true;
    }

    @Override
    public void onMicroblocksChanged() {

    }

    @Override
    public boolean hasCapability(Capability<?> capability, EnumFacing facing) {

        if (super.hasCapability(capability, facing)) return true;
        return MultipartCapabilityHelper.hasCapability(container, capability, facing);
    }

    @Override
    public <T> T getCapability(Capability<T> capability, EnumFacing facing) {

        T impl = super.getCapability(capability, facing);
        if (impl != null) return impl;
        return MultipartCapabilityHelper.getCapability(container, capability, facing);
    }

    @Override
    public boolean hasCapability(Capability<?> capability, PartSlot slot, EnumFacing facing) {

        return container.hasCapability(capability, slot, facing);
    }

    @Override
    public <T> T getCapability(Capability<T> capability, PartSlot slot, EnumFacing facing) {

        return container.getCapability(capability, slot, facing);
    }

    @Override
    public void onLoad() {

        super.onLoad();
        for (IMultipart part : getMicroblockContainer().getParts())
            part.onLoaded();
    }

    @Override
    public void onChunkUnload() {

        super.onChunkUnload();
        for (IMultipart part : getMicroblockContainer().getParts())
            part.onUnloaded();
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound compound) {

        compound = super.func_189515_b(compound);
        getMicroblockContainer().getPartContainer().writeToNBT(compound);
        return compound;
    }

    @Override
    public void func_145839_a(NBTTagCompound compound) {

        super.func_145839_a(compound);
        getMicroblockContainer().getPartContainer().readFromNBT(compound);
    }

    @Override
    public NBTTagCompound func_189517_E_() {

        return getMicroblockContainer().getPartContainer().writeToNBT(super.func_189517_E_());
    }

    @Override
    public SPacketUpdateTileEntity func_189518_D_() {

        return new SPacketUpdateTileEntity(func_174877_v(), func_145832_p(), func_189517_E_());
    }

    @Override
    public void onDataPacket(NetworkManager net, SPacketUpdateTileEntity pkt) {

        func_145839_a(pkt.func_148857_g());
    }

    @Override
    public boolean canRenderBreaking() {

        return true;
    }

    @Override
    public boolean shouldRenderInPass(int pass) {

        return true;
    }

    @Override
    public AxisAlignedBB getRenderBoundingBox() {

        AxisAlignedBB bounds = super.getRenderBoundingBox().func_72317_d(-getPosIn().func_177958_n(), -getPosIn().func_177956_o(), -getPosIn().func_177952_p());
        for (IMultipart part : getMicroblockContainer().getParts()) {
            AxisAlignedBB bb = part.getRenderBoundingBox();
            if (bb != null) bounds = bounds.func_111270_a(bb);
        }
        return bounds.func_72317_d(getPosIn().func_177958_n(), getPosIn().func_177956_o(), getPosIn().func_177952_p());
    }
}
